import System.IO
import System.Random
import Data.List
import Data.Char (isSpace)
import Control.Monad

-- 
type Chooser = State ([Section],[Section],StdGen)

-- Trims leading and trailing whitespace
trim :: String -> String
trim = f . f
   where f = reverse . dropWhile isSpace

-- Splits a string by a delimiter (here, ',')
splitByComma :: String -> [String]
splitByComma [] = [""]
splitByComma (c:cs)
  | c == ','  = "" : rest
  | otherwise = (c : head rest) : tail rest
  where rest = splitByComma cs

-- Parses a bracketed list string into a list of strings
parseList :: String -> [String]
parseList str = map trim $ splitByComma $ init $ tail str

-- Example usage
example :: [String]
example = parseList "[str1,str2]"

data Section = Section (Maybe String) -- Id for the section
                       [String] -- List of dependencies
                       (Maybe Float)
                       String -- text of section
               deriving (Eq,Show)

findKey :: String -> [String] -> Maybe String -- unparsed data extracted
findKey k ss = case findIndex (== k) ss of
                 Just i -> Just (ss !! (i+1))
                 Nothing -> Nothing

hasTitle :: Section -> Bool
hasTitle (Section (Just _) _ _ _) = True
hasTitle _ = False

-- only works if hasTitle filter has been run
extractTitle :: Section -> String
extractTitle (Section (Just t) _ _ _) = t
extractTitle _ = error "Should be impossible"

parseHeader :: String -> (String -> Section) -- the partial section
parseHeader h = \s -> Section title deps prob s 
 where splitHeader = drop 1 $ words h
       title = findKey ":title" splitHeader
       deps = case findKey ":deps" splitHeader of
                Nothing -> []
                Just x -> (parseList x)
       prob = fmap read (findKey ":prob" splitHeader)


collectSections :: [String] -> [Section]
collectSections [] = []
collectSections (x : xs) | isBegin x = let seccer = parseHeader x
                                           body = takeWhile (not . isEnd) xs
                                           rest = drop 1 $ dropWhile (not . isEnd) xs
                                       in (seccer $ unlines body) : (collectSections rest)
                         | otherwise = collectSections xs

isBegin s = take (length start) s == start
 where start = "begin-section"

isEnd s = take (length end) s == end
 where end = "end-section"

parseFile :: String -> [Section]
parseFile = collectSections . lines

-- remove all the examples that don't pass the dice roll 
rollProbs :: (RandomGen g) => g -> [Section] -> [Section]
rollProbs g [] = []
rollProbs g ((Section t ds (Just p) body) : xs) = let (n,g') = uniformR (0,1::Float) g
                                                    in if n < p
                                                       then (Section t ds (Just p) body) : (rollProbs g' xs)
                                                       else rollProbs g' xs
rollProbs g (x : xs) = x : (rollProbs g xs)



depsExist :: Section -> [Section] -> Bool
depsExist (Section _ ds _ b) ss = all aux ds
 where titles = map extractTitle $ filter hasTitle ss
       aux s = case find (==s) titles of
                 Just _ -> True
                 Nothing -> False

{- 
 Okay so we need to finish this by
 + checking that the dependencies are met at every step
 + checking that exclusions are also met 
-}
randomizeSections :: (RandomGen g) => g -> [Section] -> [Section]
randomizeSections g [] = []
randomizeSections g [x] = [x]
randomizeSections g xs = undefined

main :: IO ()
main = do
  contents <- readFile "test2.txt"
  g <- initStdGen
  let parsed = parseFile contents
      p = rollProbs g parsed
      ss = filter (\x -> depsExist x p) p 
  print $ ss
  
